/**
 * Compares two variables with the given operator or evaluates one variable in case of `isFalsy`.
 * TODO: Check if `isFalsy` should better be in a separated helper
 *
 * @param {String|Number} lval  - first value
 * @param {String} operator     - See function below for supported operator types
 * @param {String|Number} rval  - second value
 * @returns {boolean} comparison result
 */
function compare(lval, operator, rval) {
  switch (operator) {
  case '==':
    return lval == rval; // eslint-disable-line eqeqeq
  case '!=':
    return lval != rval; // eslint-disable-line eqeqeq
  case '===':
    return lval === rval;
  case '!==':
    return lval !== rval;
  case '<':
    return lval < rval;
  case '<=':
    return lval <= rval;
  case '>':
    return lval > rval;
  case '>=':
    return lval >= rval;
  case '||':
    return lval || rval;
  case '&&':
    return lval && rval;
  case 'startsWith':
  case 'starts with':
    return (bothTypeOf(lval, rval, 'string') && lval.slice(0, rval.length) === rval);
  case 'endsWith':
  case 'ends with':
    return (bothTypeOf(lval, rval, 'string') && lval.slice(-rval.length) === rval);
  case 'isFalsy':
    return !lval;
  default:
    throw new Error(`Unknown compare operator ${operator}`);
  }
}


/**
 * Checks the type of two variables and only returns true if both variables are of the expected type
 *
 * @param {*} lval      -
 * @param {*} rval      -
 * @param {String} type - Result of a typeof comparison (e.g. 'object')
 * @returns {boolean}   - true if both variables have the expected type
 */
function bothTypeOf(lval, rval, type) {
  return typeof lval === type && typeof rval === type; // eslint-disable-line valid-typeof
}


module.exports = function ife(lval, operator, rval, options) {
  if (compare(lval, operator, rval)) {
    return options.fn(this);
  }

  return options.inverse(this);
};
