const path = require('path');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const CleanWebpackPlugin = require('clean-webpack-plugin');
const previewPlugins = require('./preview/webpack.plugins');

const PREVIEW = process.env.PREVIEW === 'true';
const OPEN = process.env.OPEN === 'true';
const { version: designVersion } = require('./package.json');

module.exports = {
  entry: { 
    page: './src/page.js',
    editor: './src/editor.js' 
  },
  output: {
    path: path.resolve(__dirname, 'dist'),
    filename: '[name].js'
  },
  module: {
    rules: [
      {
        test: /\.js$/,
        exclude: /node_modules/,
        use: {
          loader: "babel-loader"
        }
      },
      {
        test: /\.scss$/,
        use: [
          { loader: "style-loader" },
          { loader: MiniCssExtractPlugin.loader },
          { loader: "css-loader" },
          { loader: "postcss-loader" },
          { loader: "sass-loader" }
        ]
      },
      {
        test: /\.(woff|woff2)$/,
        use: {
          loader: 'file-loader',
          options: {
            limit: 0,
            name: './assets/fonts/[name].[ext]'
          }
        }
      },
      {
        test: /\.(png|jpe?g|svg|gif|ico)$/,
        use: {
          loader: 'url-loader',
          options: {
            limit: 3200, // inline everything under 3200 bytes
            name: './assets/images/[name].[ext]'
          }
        }
      }
    ]
  },
  plugins: [
    new CleanWebpackPlugin('dist', { exclude: ['design.js', 'design.json']} ),
    new MiniCssExtractPlugin({
      filename: "[name].css"
    })
  ].concat(
    PREVIEW ? previewPlugins({ dest: path.resolve(__dirname, 'dist') }) : []
  ),

  devServer: {
    allowedHosts: [
      'localhost'
    ],
    disableHostCheck: true, // enables access from other devices on the host ip
    contentBase: [
      path.join(__dirname, 'dist'), 
      path.join(__dirname, 'preview', 'preview-assets')
    ],
    compress: true,

    headers: {
      'Access-Control-Allow-Origin': '*'
    },

    // host: '0.0.0.0',
    // https://webpack.js.org/concepts/hot-module-replacement/
    hot: false, // important to be false. ohterwise nothing works in szm-delivery...
    open: OPEN,
    overlay: true,
    port: 3334,

    // make assets available under e.g. http://localhost:9000/szm/2.0.0/
    publicPath: `/szdigital/${designVersion}/`,
    openPage: `szdigital/${designVersion}/`
  }
};